package it.inaf.oats.ia2.tapschemamanager.businesslayer;

import it.inaf.oats.ia2.tapschemamanager.datalayer.SchemaEntity;
import it.inaf.oats.ia2.tapschemamanager.datalayer.TableEntity;
import it.inaf.oats.ia2.tapschemamanager.datalayer.TapSchemaHandler;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 *
 * @author Sonia Zorba <zorba at oats.inaf.it>
 */
public class Schema extends EntityWrapper implements EntityWrapperContainer {

    public static final String UTYPE = "utype";
    public static final String DESCRIPTION = "description";

    private final TapSchemaHandler tapSchemaHandler;

    private String selectedTable;
    private final Map<String, Table> tables;

//    public Schema(TapSchemaHandler tapSchemaHandler, String schemaName) throws SQLException {
//        this(tapSchemaHandler, tapSchemaHandler.getNewSchema(schemaName));
//        setStatus(Status.ADDED_NOT_PERSISTED);
//    }
    public Schema(TapSchemaHandler tapSchemaHandler, SchemaEntity schemaEntity) throws SQLException {
        super(schemaEntity, UTYPE, DESCRIPTION);

        this.tapSchemaHandler = tapSchemaHandler;

        addValue(UTYPE, schemaEntity.getUtype());
        addValue(DESCRIPTION, schemaEntity.getDescription());

        tables = new HashMap<String, Table>();

        for (TableEntity tableEntity : schemaEntity.getTables().values()) {
            String tableName = tableEntity.getTableName();
            tables.put(tableName, new Table(tapSchemaHandler, getName(), tableName, tableEntity));
        }

        setStatus(Status.ADDED_PERSISTED);
    }

    @Override
    protected void afterSetValue(String key, String value) {
        SchemaEntity schemaEntity = getSchemaEntity();
        if (key.equals(UTYPE)) {
            schemaEntity.setUtype(value);
        } else if (key.equals(DESCRIPTION)) {
            schemaEntity.setDescription(value);
        }
    }

    public List<String> getTablesNames() {
        return Util.setToList(tables.keySet());
    }

    @Override
    public List<String> getAddables() {
        return Util.getAddables(tapSchemaHandler.getAllTables(getName()), tables.keySet());
    }

    @Override
    public void addEntityWrapper(String name) throws SQLException {
        Table table = tables.get(name);
        if (table == null) {
            // Add new 
            TableEntity tableEntity = tapSchemaHandler.getNewTable(getName(), name);
            table = new Table(tapSchemaHandler, getName(), name, tableEntity);
            tables.put(name, table);
            table.setStatus(Status.ADDED_NOT_PERSISTED);
        } else {
            // Undo remove
            table.setStatus(Status.ADDED_PERSISTED);
        }
        selectedTable = name;

        getSchemaEntity().addTable((TableEntity) table.getEntity());
    }

    @Override
    public void removeEntityWrapper(String name) {
        Table table = tables.get(name);
        if (table.getStatus() == Status.ADDED_NOT_PERSISTED) {
            tables.remove(name);
        } else {
            table.setStatus(Status.TO_REMOVE);
        }
        selectedTable = tables.isEmpty() ? null : tables.keySet().iterator().next();

        getSchemaEntity().getTables().remove(table.getFullName());
    }

    public Table getTable(String name) {
        return tables.get(name);
    }

    protected SchemaEntity getSchemaEntity() {
        return (SchemaEntity) getEntity();
    }

    @Override
    protected void setPersisted() {
        super.setPersisted();
        Iterator<Table> iterator = tables.values().iterator();
        while (iterator.hasNext()) {
            Table table = iterator.next();
            if (table.getStatus() == Status.TO_REMOVE) {
                iterator.remove();
            } else {
                table.setPersisted();
            }
        }
    }

    @Override
    public boolean hasChildren() {
        return !tables.isEmpty();
    }

    @Override
    public List<EntityWrapper> getAllEntityWrappers() {
        return new ArrayList(tables.values());
    }

    public Table getSelectedTable() {
        return selectedTable == null ? null : tables.get(selectedTable);
    }

    @Override
    public void selectEntity(String entityName) {
        selectedTable = entityName;
        Table table = tables.get(entityName);
        List<String> columns = table.getColumnsNames();
        if (!columns.isEmpty()) {
            table.selectEntity(columns.get(0));
        }
    }

    @Override
    public EntityWrapper getSelectedEntity() {
        if (selectedTable == null) {
            return null;
        }
        return tables.get(selectedTable);
    }
}
