package it.inaf.oats.ia2.tapschemamanager.businesslayer;

import it.inaf.oats.ia2.tapschemamanager.datalayer.ColumnEntity;
import it.inaf.oats.ia2.tapschemamanager.datalayer.ColumnInfo;
import it.inaf.oats.ia2.tapschemamanager.datalayer.TableEntity;
import it.inaf.oats.ia2.tapschemamanager.datalayer.TapSchemaHandler;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 *
 * @author Sonia Zorba <zorba at oats.inaf.it>
 */
public class Table extends EntityWrapper implements EntityWrapperContainer {

    public static final String UTYPE = "utype";
    public static final String DESCRIPTION = "description";

    private String selectedColumn;
    private final Map<String, Column> columns;

    private final String schemaName;
    private final String tableName;

//    public Table(TapSchemaHandler tapSchemaHandler, String schemaName, String tableName) throws SQLException {
//        this(tapSchemaHandler, schemaName, tableName, new TableEntity(schemaName + "." + tableName));
//        setStatus(Status.ADDED_NOT_PERSISTED);
//    }
    public Table(TapSchemaHandler tapSchemaHandler, String schemaName, String tableName, TableEntity tableEntity) throws SQLException {
        super(tableEntity, UTYPE, DESCRIPTION);

        addValue(UTYPE, tableEntity.getUtype());
        addValue(DESCRIPTION, tableEntity.getDescription());

        this.schemaName = schemaName;
        this.tableName = tableName;

        // Load all ColumnEntity map
        columns = new HashMap<String, Column>();
        Set<String> alreadyLoadedColumns = tableEntity.getColumns().keySet();
        for (ColumnInfo columnInfo : tapSchemaHandler.getColumnInfo(schemaName, tableName)) {
            ColumnEntity columnEntity = columnInfo.getColumnEntity();
            Column column = new Column(columnEntity, columnInfo.isPrimaryKey());
            columns.put(columnEntity.getName(), column);

            if (alreadyLoadedColumns.contains(columnEntity.getName())) {
                column.setHidden(false);
                column.setStatus(Status.ADDED_PERSISTED);
            }
        }

        // Load foreign keys constraints
        // TODO ...
        setStatus(Status.ADDED_PERSISTED);
    }

    @Override
    protected void afterSetValue(String key, String value) {
        TableEntity tableEntity = getTableEntity();
        if (key.equals(UTYPE)) {
            tableEntity.setUtype(value);
        } else if (key.equals(DESCRIPTION)) {
            tableEntity.setDescription(value);
        }
    }

    protected TableEntity getTableEntity() {
        return (TableEntity) getEntity();
    }

    @Override
    public List<String> getAddables() {
        List<String> addables = new ArrayList<String>();
        for (Column column : columns.values()) {
            if (column.isHidden()) {
                addables.add(column.getName());
            }
        }
        return addables;
    }

    @Override
    public void addEntityWrapper(String name) throws SQLException {
        Column column = columns.get(name);
        if (column.isHidden()) {
            // Add new
            column.setStatus(Status.ADDED_NOT_PERSISTED);
            column.setHidden(false);
        } else {
            // Undo remove
            column.setStatus(Status.ADDED_PERSISTED);
        }
        selectedColumn = name;

        getTableEntity().addColumn(column.getColumnEntity());
    }

    @Override
    public void removeEntityWrapper(String name) {
        Column column = columns.get(name);

        if (column.getStatus() == Status.ADDED_NOT_PERSISTED) {
            column.setHidden(true);
        } else {
            column.setStatus(Status.TO_REMOVE);
        }
        List<String> visibleColumns = getColumnsNames();
        selectedColumn = visibleColumns.isEmpty() ? null : visibleColumns.get(0);

        column.setStatus(Status.ADDED_NOT_PERSISTED);
        getTableEntity().addColumn(column.getColumnEntity());
    }

    public Column getColumn(String name) {
        return columns.get(name);
    }

    public List<String> getColumnsNames() {
        List<String> columnsNames = new ArrayList<String>();
        for (Column column : columns.values()) {
            if (!column.isHidden()) {
                columnsNames.add(column.getName());
            }
        }
        return columnsNames;
    }

    @Override
    protected void setPersisted() {
        super.setPersisted();
        for (Column column : columns.values()) {
            if (column.getStatus() == Status.TO_REMOVE) {
                column.setHidden(true);
            } else {
                column.setPersisted();
            }
        }
    }

    @Override
    public boolean hasChildren() {
        for (Column column : columns.values()) {
            if (!column.isHidden()) {
                return true;
            }
        }
        return false;
    }

    @Override
    public List<EntityWrapper> getAllEntityWrappers() {
        List<EntityWrapper> list = new ArrayList<EntityWrapper>();
        for (Column column : columns.values()) {
            if (!column.isHidden()) {
                list.add(column);
            }
        }
        return list;
    }

    public Column getSelectedColumn() {
        return selectedColumn == null ? null : columns.get(selectedColumn);
    }

    @Override
    public void selectEntity(String entityName) {
        selectedColumn = entityName;
    }

    public String getSchemaName() {
        return schemaName;
    }

    public String getFullName() {
        return getEntity().getName();
    }

    @Override
    public String getName() {
        return tableName;
    }

    @Override
    public EntityWrapper getSelectedEntity() {
        if (selectedColumn == null) {
            return null;
        }
        return columns.get(selectedColumn);
    }
}
