package it.inaf.oats.ia2.tapschemamanager.webapp;

import it.inaf.oats.ia2.tapschemamanager.businesslayer.Column;
import it.inaf.oats.ia2.tapschemamanager.businesslayer.EntityWrapper;
import it.inaf.oats.ia2.tapschemamanager.businesslayer.EntityWrapperContainer;
import it.inaf.oats.ia2.tapschemamanager.businesslayer.Schema;
import it.inaf.oats.ia2.tapschemamanager.businesslayer.Status;
import it.inaf.oats.ia2.tapschemamanager.businesslayer.Table;
import it.inaf.oats.ia2.tapschemamanager.businesslayer.TapSchema;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 *
 * @author Sonia Zorba <zorba at oats.inaf.it>
 */
public class EditTapSchemaPageModel {

    class Entity {

        boolean toRemove;
        Map<String, EditableField> values;

        Entity(EntityWrapper dbEntity) {
            toRemove = dbEntity.getStatus() == Status.TO_REMOVE;
            values = getEditablesFields(dbEntity);
        }
    }

    class EditableField {

        Object value;
        boolean isChanged;

        EditableField(String key, EntityWrapper entityWrapper) {
            this.value = entityWrapper.getValue(key);
            this.isChanged = entityWrapper.isChanged(key);
        }
    }

    class EntityItem {

        private final String title;
        private final boolean toRemove;
        private final boolean active;

        public EntityItem(EntityWrapper entityWrapper, boolean active) {
            this.title = entityWrapper.getName();
            this.toRemove = entityWrapper.getStatus() == Status.TO_REMOVE;
            this.active = active;
        }
    }

    class TapSchemaModel {

        boolean hasAddables;
        boolean hasChildren;
    }

    class SchemaModel extends Entity {

        boolean hasAddables;
        boolean hasChildren;

        public SchemaModel(Schema schema) {
            super(schema);
            if (schema != null) {
                hasAddables = !schema.getAddables().isEmpty();
                hasChildren = !schema.hasChildren();
            }
        }
    }

    class TableModel extends Entity {

        boolean hasAddables;
        boolean hasChildren;

        public TableModel(Table table) {
            super(table);
            if (table != null) {
                hasAddables = !table.getAddables().isEmpty();
                hasChildren = !table.hasChildren();
            }
        }
    }

    class ColumnModel extends Entity {

        boolean isPrimaryKey;
        boolean isForeignKey;
        String dataType;
        int size;

        ColumnModel(Column column) {
            super(column);

            isPrimaryKey = column.isPrimaryKey();
            isForeignKey = column.isForeignKey();
            dataType = column.getDatatype();
            size = column.getSize();
        }
    }

    TapSchemaModel tapSchema;

    SchemaModel schema;
    TableModel table;
    ColumnModel column;

    List<EntityItem> schemas;
    List<EntityItem> tables;
    List<EntityItem> columns;

    private transient EditTapSchemaPageState pageState;

    public EditTapSchemaPageModel(EditTapSchemaPageState pageState) {

        TapSchema ts = pageState.getTapSchema();

        Schema selectedSchema = ts.getSelectedSchema();
        Table selectedTable = ts.getSelectedTable();
        Column selectedColumn = ts.getSelectedColumn();

        this.tapSchema = new TapSchemaModel();
        this.tapSchema.hasAddables = !ts.getAddables().isEmpty();
        this.tapSchema.hasChildren = !ts.getAllEntityWrappers().isEmpty();

        schemas = getAllEntities(ts, selectedSchema);
        tables = getAllEntities(selectedSchema, selectedTable);
        columns = getAllEntities(selectedTable, selectedColumn);

        schema = selectedSchema == null ? null : new SchemaModel(selectedSchema);
        table = selectedTable == null ? null : new TableModel(selectedTable);
        column = selectedColumn == null ? null : new ColumnModel(selectedColumn);
    }

    private List<EntityItem> getAllEntities(EntityWrapperContainer container, EntityWrapper selectedEntityWrapper) {
        if (container == null) {
            return null;
        }

        ArrayList<EntityItem> entities = new ArrayList<EntityItem>();
        String selectedName = selectedEntityWrapper == null ? null : selectedEntityWrapper.getName();
        for (EntityWrapper entityWrapper : container.getAllEntityWrappers()) {
            entities.add(new EntityItem(entityWrapper, entityWrapper.getName().equals(selectedName)));
        }
        return entities;
    }

    private Map<String, EditableField> getEditablesFields(EntityWrapper entityWrapper) {
        if (entityWrapper == null) {
            return null;
        }

        Map<String, EditableField> fields = new HashMap<String, EditableField>();
        for (String key : entityWrapper.getSupportedKeys()) {
            fields.put(key, new EditableField(key, entityWrapper));
        }
        return fields;
    }

    public void updateEditableField(String entityName, String key, String value, boolean isChanged) {
        Entity entity = null;
        if (entityName.equals("schema")) {
            entity = schema;
        } else if (entityName.equals("table")) {
            entity = table;
        } else if (entityName.equals("column")) {
            entity = column;
        }
        EditableField editableField = entity.values.get(key);
        editableField.value = value;
        editableField.isChanged = isChanged;
    }
}
