package it.inaf.oats.ia2.tapschemamanager.datalayer;

import java.io.Serializable;
import java.math.BigInteger;
import java.util.HashMap;
import java.util.Map;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.MapKeyColumn;
import javax.persistence.OneToMany;
import javax.persistence.Table;

/**
 *
 * @author Sonia Zorba <zorba at oats.inaf.it>
 */
@Entity
@Table(name = "`schemas`")
public class SchemaEntity implements Serializable, TapSchemaEntity {

    @Id
    @Column(name = "schema_name", length = 64)
    private String name;

    @Column(name = "utype", length = 512)
    private String utype;

    @Column(name = "description", length = 512)
    private String description;

    @Column(name = "schemaID")
    private BigInteger schemaID;

    @OneToMany(mappedBy = "schema", cascade = CascadeType.ALL, orphanRemoval = true)
    @MapKeyColumn(name = "table_name", insertable = false, updatable = false)
    private final Map<String, TableEntity> tables;

    protected SchemaEntity() {
        this.tables = new HashMap<String, TableEntity>();
    }

    protected SchemaEntity(String schemaName) {
        this();
        this.name = schemaName;
    }

    @Override
    public String getName() {
        return name;
    }

    protected void setName(String schemaName) {
        this.name = schemaName;
    }

    public String getUtype() {
        return utype;
    }

    public void setUtype(String utype) {
        this.utype = utype;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    protected BigInteger getSchemaID() {
        return schemaID;
    }

    protected void setSchemaID(BigInteger schemaID) {
        this.schemaID = schemaID;
    }

    public Map<String, TableEntity> getTables() {
        return tables;
    }

    public void addTable(TableEntity table) {
        tables.put(table.getName(), table);
        table.setSchemaName(name);
    }
}
