package it.inaf.oats.ia2.tapschemamanager.datalayer;

import java.io.Serializable;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Pattern;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.Id;
import javax.persistence.ManyToOne;
import javax.persistence.MapKeyColumn;
import javax.persistence.OneToMany;
import javax.persistence.PrimaryKeyJoinColumn;
import javax.persistence.Table;

/**
 *
 * @author Sonia Zorba <zorba at oats.inaf.it>
 */
@Entity
@Table(name = "tables")
public class TableEntity implements Serializable, TapSchemaEntity {

    @Id
    @Column(name = "table_name", length = 128)
    private String name;

    @Column(name = "schema_name", length = 64)
    private String schemaName;

    @Column(name = "table_type", length = 8)
    private String tableType;

    @Column(name = "utype", length = 512)
    private String utype;

    @Column(name = "description", length = 512)
    private String description;

    @Column(name = "tableID")
    private BigInteger tableID;

    @ManyToOne(fetch = FetchType.EAGER)
    @PrimaryKeyJoinColumn(name = "schema_name", referencedColumnName = "schema_name")
    private SchemaEntity schema;

    @OneToMany(mappedBy = "table", cascade = CascadeType.ALL, orphanRemoval = true)
    @MapKeyColumn(name = "column_name", insertable = false, updatable = false)
    private final Map<String, ColumnEntity> columns;

    @OneToMany(mappedBy = "fromTable", cascade = CascadeType.ALL, orphanRemoval = true)
    private final List<KeyEntity> fromKeys;

    @OneToMany(mappedBy = "targetTable", cascade = CascadeType.ALL, orphanRemoval = true)
    private final List<KeyEntity> targetKeys;

    protected TableEntity() {
        columns = new HashMap<String, ColumnEntity>();
        fromKeys = new ArrayList<KeyEntity>();
        targetKeys = new ArrayList<KeyEntity>();
    }

    protected TableEntity(String tableName) {
        this();
        this.name = tableName;
    }

    /**
     * @return full table name: schemaName.tableName
     */
    @Override
    public String getName() {
        return name;
    }

    /**
     * @return table name without schema name
     */
    public String getTableName() {
        return name.split(Pattern.quote("."))[1];
    }

    protected void setName(String tableName) {
        this.name = tableName;
    }

    public String getSchemaName() {
        return schemaName;
    }

    protected void setSchemaName(String schemaName) {
        this.schemaName = schemaName;
    }

    public String getTableType() {
        return tableType;
    }

    public void setTableType(String tableType) {
        this.tableType = tableType;
    }

    public String getUtype() {
        return utype;
    }

    public void setUtype(String utype) {
        this.utype = utype;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    protected BigInteger getTableID() {
        return tableID;
    }

    protected void setTableID(BigInteger tableID) {
        this.tableID = tableID;
    }

    public SchemaEntity getSchema() {
        return schema;
    }

    public Map<String, ColumnEntity> getColumns() {
        return columns;
    }

    public List<KeyEntity> getFromKeys() {
        return fromKeys;
    }

    public List<KeyEntity> getTargetKeys() {
        return targetKeys;
    }

    public void addColumn(ColumnEntity column) {
        columns.put(column.getName(), column);
        column.setTableName(name);
    }
}
