
import it.inaf.oats.ia2.tapschemamanager.datalayer.ColumnEntity;
import it.inaf.oats.ia2.tapschemamanager.datalayer.Credentials;
import it.inaf.oats.ia2.tapschemamanager.datalayer.SchemaEntity;
import it.inaf.oats.ia2.tapschemamanager.datalayer.TableEntity;
import it.inaf.oats.ia2.tapschemamanager.datalayer.TapSchemaHandler;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.List;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import static org.junit.Assert.*;
import org.junit.Ignore;

/**
 *
 * @author Sonia Zorba <zorba at oats.inaf.it>
 */
public class TestQuery {

    private static final int DATABASES_COUNT = 2;
    private static final int TABLES_COUNT = 3;
    private static Credentials credentials;

    public TestQuery() {
    }

    @BeforeClass
    public static void setUpClass() {
        credentials = new Credentials();
        credentials.setHostname("localhost");
        credentials.setUsername("root");
        credentials.setPassword("root");
    }

    @AfterClass
    public static void tearDownClass() {
    }

    @Before
    public void setUp() {
    }

    @After
    public void tearDown() {
    }

    private void removeTestingDatabases(Credentials credentials) throws SQLException {
        Connection connection = credentials.getConnection();
        Statement statement = connection.createStatement();
        for (int i = 0; i < DATABASES_COUNT; i++) {
            statement.executeUpdate("DROP DATABASE IF EXISTS db" + i);
        }
        statement.executeUpdate("DROP DATABASE IF EXISTS test_tap_schema");
        statement.close();
        connection.close();
    }

    private void setUpTestingDatabases(Credentials credentials) throws SQLException {
        removeTestingDatabases(credentials);

        Connection connection = credentials.getConnection();

        Statement statement = connection.createStatement();
        for (int i = 0; i < DATABASES_COUNT; i++) {
            statement.executeUpdate("CREATE DATABASE db" + i);
            for (int j = 0; j < TABLES_COUNT; j++) {
                statement.executeUpdate("CREATE TABLE db" + i + ".table" + j + " (\n"
                        + "id INT PRIMARY KEY AUTO_INCREMENT,\n"
                        + "value1 VARCHAR(255),\n"
                        + "value2 FLOAT\n"
                        + ");");
            }
        }
        statement.close();

        connection.setCatalog("db0");

        statement = connection.createStatement();
        for (int j = 0; j < TABLES_COUNT - 1; j++) {
            statement.executeUpdate("ALTER TABLE table" + (j + 1) + " ADD COLUMN table" + j + "_id INT");
            statement.executeUpdate("ALTER TABLE table" + (j + 1) + " ADD CONSTRAINT id_constraint FOREIGN KEY(table" + j + "_id) REFERENCES table" + j + "(id)");
        }
        statement.close();

        connection.close();
    }

    @Test
    public void loadSchema() throws SQLException {
        TapSchemaHandler tapSchemaHandler = new TapSchemaHandler(credentials, "vlkb_voinfo_schema", true);
        assertTrue(!tapSchemaHandler.getSchemas().isEmpty());
    }

    @Test
    public void createNewAndUpdate() throws SQLException {
        System.out.println("TEST createNewAndUpdate STARTED");

        try {
            setUpTestingDatabases(credentials);

            // TEST first TapSchema creation
            TapSchemaHandler tsh = new TapSchemaHandler(credentials, "test_tap_schema", false);

            SchemaEntity db0 = tsh.getNewSchema("db0");
            tsh.addSchema(db0);

            // Test add, remove and re-add of a table
            List<String> db0Tables = tsh.getAllTables(db0.getName());

            TableEntity db0table0 = tsh.getNewTable(db0.getName(), "table0");
            assertEquals(db0table0.getName(), "db0.table0");
            db0.addTable(db0table0);
            TableEntity db0table1 = tsh.getNewTable(db0.getName(), "table1");
            
            //db0table1.getFromKeys();
            
            
            tsh.save();
            assertNotNull(db0.getTables().get("db0.table0"));

            tsh = new TapSchemaHandler(credentials, "test_tap_schema", true);
            db0 = tsh.getSchemas().iterator().next();
            db0.getTables().remove("db0.table0");
            assertNull(db0.getTables().get("db0.table0"));
            tsh.save();

            tsh = new TapSchemaHandler(credentials, "test_tap_schema", true);
            db0 = tsh.getSchemas().iterator().next();
            assertNull(db0.getTables().get("db0.table0"));

            // Test key constraints
            db0.addTable(db0table0);
            
            
//            assertEquals(db0Tables.size(), TABLES_COUNT);
//            for (String tableName : db0Tables) {
//                tapSchemaHandler.get
//            }
//            
//            TableEntity db0_
//            
//            List<String> addables = tapSchema.getAddables();
//            assertTrue(addables.contains("db0"));
//            assertTrue(addables.contains("db1"));
//
//            tapSchema.addEntityWrapper("db0");
//            assertFalse(tapSchema.getAddables().contains("db0"));
//
//            Schema schema = tapSchema.getSchema("db0");
//
//            addables = schema.getAddables();
//            for (int i = 0; i < addables.size(); i++) {
//                String tableName = addables.get(i);
//                assertEquals(tableName, "table" + i);
//                schema.addEntityWrapper(tableName);
//            }
//            assertTrue(schema.getAddables().isEmpty());
//
//            Table table1 = schema.getTable("table1");
//            assertEquals(table1.getName(), "table1");
//            assertEquals(table1.getFullName(), "db0.table1");
//            table1.addEntityWrapper("id");
//            table1.addEntityWrapper("value1");
//            assertFalse(table1.getAddables().contains("id"));
//            assertTrue(table1.getAddables().contains("value2"));
//
//            Column column = table1.getColumn("id");
//            column.setValue(Column.DESCRIPTION, "Test description");
//            assertTrue(column.isChanged());
//            column.setValue(Column.DESCRIPTION, "");
//            assertFalse(column.isChanged());
        } catch (SQLException e) {
            throw e;
        } finally {
            removeTestingDatabases(credentials);
        }
    }

    @Ignore
    @Test
    public void query() throws SQLException {

//        EntityManagerFactory emf = Persistence.createEntityManagerFactory("pu");
//        EntityManager em = emf.createEntityManager();
        //CriteriaBuilder cb = em.getCriteriaBuilder();
        //CriteriaQuery<SchemaEntity> cq = cb.createQuery(SchemaEntity.class);
        //List<SchemaEntity> results = em.createQuery(cq).getResultList();
//        SchemaEntity schema = new SchemaEntity("test_schema");
//
//        TableEntity table = new TableEntity("table1");
//        schema.addTable(table);
//
//        ColumnEntity column = new ColumnEntity("table1_column1");
//        table.addColumn(column);

        //TapSchemaHandler tapSchemaHandler = new TapSchemaHandler(credentials, "TestJPA", false);
        //statement = connection.createStatement();
        //    String tapschemaName = tapSchema.getName();
        //creates database
        //String queryString = "CREATE DATABASE IF NOT EXISTS " + tapschemaName + ";";
        //statement.executeUpdate(queryString);
        //em.getTransaction().begin();
//        for (SchemaEntity schema : results) {
//            System.out.println("schema = " + schema.getSchemaName());
//            for (TableEntity table : schema.getTables().values()) {
//                System.out.println("table = " + table.getTableName());
//            }
//
////            if (schema.getSchemaName().equals("geo")) {
////                Map<String, TableEntity> tables = schema.getTables();
////                TableEntity table = schema.getTables().get("geo.country_block_ipv4");
////                //em.remove(table);
//////                for (ColumnEntity column : table.getColumns().values()) {
//////                    System.out.println("column = " + column.getColumnName());
//////                    column.setDescription("testDescription");
//////                }
////                break;
////            }
//        }
        //em.getTransaction().commit();
        //results.get(i)
//        for (SchemaEntity schema : results) {
//            System.out.println("schema = " + schema.getSchemaName());
//            for (TableEntity table : schema.getTables()) {
//                System.out.println("table = " + table.getTableName());
//                for (ColumnEntity column : table.getColumns()) {
//                    System.out.println("column = " + column.getColumnName());
//                }
//                for (KeyEntity fromKey : table.getFromKeys()) {
//                    System.out.println("fromKey = " + fromKey.getKeyId());
//                }
//                for (KeyEntity targetKey : table.getTargetKeys()) {
//                    System.out.println("targetKey = " + targetKey.getKeyId());
//                }
//            }
//        }
    }
}
